/*
This file is part of video3x.

video3x is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

video3x is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with video3x.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2011-2012 Vadim Druzhin <cdslow@mail.ru>
*/

#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include "vi3x_modules.h"

static char const *Tag="null";

static void m_help(vi3x_help_t *h);
static int m_init(vi3x_module_t *m);
static int m_fini(vi3x_module_t *m);
static int m_image(vi3x_module_t *m, vi3x_image_t *image);
static void print_long(vi3x_module_t const *m, char const *prefix, long l);

/* Entry point */
VI3X_MAIN()
    {
    vi3x_reg_export(Tag, m_init, 0, m_help);
    }

/* Module help */
static void m_help(vi3x_help_t *h)
    {
    h->help_module(Tag, VI3X_VER_STR, "Vadim Druzhin",
        "Dummy export module, just outputs video parameters",
        "This module is useful for scripting.",
        NULL);
    }

/* Instance initialization */
static int m_init(vi3x_module_t *m)
    {
    print_long(m, "WIDTH=", m->w);
    print_long(m, "HEIGHT=", m->h);
    print_long(m, "FPS=", m->fps);
    print_long(m, "FPS_DIV=", m->fps_div);

    /* Publish module interface */
    m->fini=m_fini;
    m->process_image=m_image;

    return 1;
    }

/* Instance finalization */
static int m_fini(vi3x_module_t *m)
    {
    print_long(m, "FRAMES=", m->frame_n);

    return 1;
    }

/* Frame processor */
static int m_image(vi3x_module_t *m, vi3x_image_t *image)
    {
    (void)m;     /* Unused */
    (void)image; /* Unused */
    return 1;
    }

static void print_long(vi3x_module_t const *m, char const *prefix, long l)
    {
    char buf[sizeof(long) * 5 / 2 + 2]="0";

    m->vi3x.write(prefix, strlen(prefix));
    snprintf(buf, sizeof(buf), "%lu\n", l);
    m->vi3x.write(buf, strlen(buf));
    }

