/*
This file is part of video3x.

video3x is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

video3x is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with video3x.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2012 Vadim Druzhin <cdslow@mail.ru>
*/

#include <stddef.h>
#include <string.h>
#include <stdint.h>
#include <limits.h>
#include "vi3x_modules.h"

static char const *Tag = "setcolor";

/* Instance local data */
typedef struct m_priv_s
    {
    int fill_yy;
    int fill_cb;
    int fill_cr;
    } m_priv_t;

static int m_init(vi3x_module_t *m);
static int m_image(vi3x_module_t *m, vi3x_image_t *image);
static void m_help(vi3x_help_t *h);
static void help_yy(vi3x_arg_t const *arg, vi3x_help_t const *h);
static void help_cb(vi3x_arg_t const *arg, vi3x_help_t const *h);
static void help_cr(vi3x_arg_t const *arg, vi3x_help_t const *h);

/* Module arguments */
static vi3x_arg_t ArgMap[]=
    {
    { "yy", -1, offsetof(m_priv_t, fill_yy), help_yy },
    { "cb", -1, offsetof(m_priv_t, fill_cb), help_cb },
    { "cr", -1, offsetof(m_priv_t, fill_cr), help_cr },
    { NULL, 0, 0, NULL }
    };


/* Entry point */
VI3X_MAIN()
    {
    vi3x_reg_filter(Tag, m_init, sizeof(m_priv_t), m_help);
    }

/* Instance initialization */
static int m_init(vi3x_module_t *m)
    {
    /* Parse arguments */
    if(!m->vi3x.parse_args(m->priv, m->args, ArgMap, Tag))
        return VI3X_RET_ERR;

    /* Publish module interface */
    m->process_image=m_image;

    return VI3X_RET_OK;
    }

/* Frame processor */
static int m_image(vi3x_module_t *m, vi3x_image_t *image)
    {
    if(m->priv->fill_yy != -1)
        memset(image->yy, m->priv->fill_yy, image->plane_sz);

    if(m->priv->fill_cb != -1)
        memset(image->cb, m->priv->fill_cb, image->plane_sz);

    if(m->priv->fill_cr != -1)
        memset(image->cr, m->priv->fill_cr, image->plane_sz);

    return m->out_image(m, image);
    }

/* Module help */
static void m_help(vi3x_help_t *h)
    {
    h->help_module(Tag, VI3X_VER_STR, "Vadim Druzhin",
        "Fill color planes with given color values",
        "You can get greyscale effect with cb=128,cr=128\n"
        "and sepia effect with cb=114,cr=144.", ArgMap);
    }

static void help_yy(vi3x_arg_t const *arg, vi3x_help_t const *h)
    {
    h->help_param_range(arg->name,
        "Luma fill value",
        "'-1' means don't fill.",
        arg->def, -1, VI3X_MAX
        );
    }

static void help_cb(vi3x_arg_t const *arg, vi3x_help_t const *h)
    {
    h->help_param_range(arg->name,
        "Blue plane fill value",
        "'-1' means don't fill.",
        arg->def, -1, VI3X_MAX
        );
    }

static void help_cr(vi3x_arg_t const *arg, vi3x_help_t const *h)
    {
    h->help_param_range(arg->name,
        "Red plane fill value",
        "'-1' means don't fill.",
        arg->def, -1, VI3X_MAX
        );
    }

