/*
This file is part of video3x.

video3x is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

video3x is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with video3x.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2011-2012 Vadim Druzhin <cdslow@mail.ru>
*/

#include <stddef.h>
#include <string.h>
#include <stdint.h>
#include <limits.h>
#include "vi3x_modules.h"

static char const *Tag="timecode";

#define DIGIT_H 8
#define DIGIT_W 8
#define NUMBER_WIDTH 10

/* Instance local data */
typedef struct m_priv_s
    {
    int x;
    int y;
    } m_priv_t;

static int m_init(vi3x_module_t *m);
static int m_image(vi3x_module_t *m, vi3x_image_t *image);
static void m_help(vi3x_help_t *h);
static void help_x(vi3x_arg_t const *arg, vi3x_help_t const *h);
static void help_y(vi3x_arg_t const *arg, vi3x_help_t const *h);

static void put_digit(vi3x_image_t *image, int x, int y, int n);
static void put_pixel(vi3x_image_t *image, int x, int y,
    vi3x_sample_t yy, vi3x_sample_t cb, vi3x_sample_t cr);

/* Module arguments */
static vi3x_arg_t ArgMap[]=
    {
    { "x", DIGIT_W, offsetof(m_priv_t, x), help_x },
    { "y", DIGIT_H, offsetof(m_priv_t, y), help_y },
    { NULL, 0, 0, NULL }
    };
#define IDX_Y 1

static uint8_t const Digits[10][DIGIT_H][DIGIT_W]=
    {
    {
    {   0,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,255,255,  0,  0,  0 },
    {   0,  0,255,  0,255,  0,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,  0,  0,  0,255,255,  0,  0 },
    {   0,255,255,255,  0,  0,  0,  0 },
    { 255,  0,  0,  0,  0,  0,  0,  0 },
    { 255,255,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    { 255,255,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,255,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,255,255,  0,  0,  0 },
    {   0,  0,255,  0,255,  0,  0,  0 },
    {   0,255,  0,  0,255,  0,  0,  0 },
    { 255,255,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    { 255,255,255,255,255,255,255,  0 },
    { 255,  0,  0,  0,  0,  0,  0,  0 },
    { 255,  0,  0,  0,  0,  0,  0,  0 },
    { 255,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,  0,  0,255,255,  0,  0,  0 },
    {   0,  0,255,  0,  0,  0,  0,  0 },
    {   0,255,  0,  0,  0,  0,  0,  0 },
    { 255,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    { 255,255,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,255,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,  0,  0,255,  0,  0,  0,  0 },
    {   0,  0,255,  0,  0,  0,  0,  0 },
    {   0,255,  0,  0,  0,  0,  0,  0 },
    { 255,  0,  0,  0,  0,  0,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    },
    {
    {   0,255,255,255,255,255,  0,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    { 255,  0,  0,  0,  0,  0,255,  0 },
    {   0,255,255,255,255,255,255,  0 },
    {   0,  0,  0,  0,  0,255,  0,  0 },
    {   0,  0,  0,  0,255,  0,  0,  0 },
    {   0,255,255,255,  0,  0,  0,  0 },
    {   0,  0,  0,  0,  0,  0,  0,  0 },
    }
    };

/* Entry point */
VI3X_MAIN()
    {
    vi3x_reg_filter(Tag, m_init, sizeof(m_priv_t), m_help);
    }

/* Module help */
static void m_help(vi3x_help_t *h)
    {
    h->help_module(Tag, VI3X_VER_STR, "Vadim Druzhin",
        "Embed current frame number into the frame", "", ArgMap);
    }

static void help_x(vi3x_arg_t const *arg, vi3x_help_t const *h)
    {
    h->help_param_range(arg->name,
        "X coordinate of the text (in pixels from left)",
        "Negative values are interpreted as the offset from the right margin.",
        arg->def, INT_MIN, INT_MAX
        );
    }

static void help_y(vi3x_arg_t const *arg, vi3x_help_t const *h)
    {
    h->help_param_range(arg->name,
        "Y coordinate of the text (in pixels from top)",
        "Negative values are interpreted as the offset from the bottom margin.",
        arg->def, INT_MIN, INT_MAX
        );
    }

/* Instance initialization */
static int m_init(vi3x_module_t *m)
    {
    if(m->w < DIGIT_W * NUMBER_WIDTH)
        {
        m->vi3x.err("[%s] Error: Image width too small (%d < %d)!\n",
            Tag, m->w, DIGIT_W * NUMBER_WIDTH);
        return VI3X_RET_ERR;
        }

    if(m->h<DIGIT_H)
        {
        m->vi3x.err("[%s] Error: Image height too small (%d < %d)!\n",
            Tag, m->h, DIGIT_H);
        return VI3X_RET_ERR;
        }

    /* Parse arguments */
    if(!m->vi3x.parse_args(m->priv, m->args, ArgMap, Tag))
        return VI3X_RET_ERR;

    /* Shift default y position */
    if(m->priv->y == ArgMap[IDX_Y].def)
        ArgMap[IDX_Y].def += DIGIT_H;

    /* Correct text position */
    if(m->priv->x < 0)
        m->priv->x = m->w - DIGIT_W * NUMBER_WIDTH + m->priv->x;
    if(m->priv->x < 0)
        m->priv->x = 0;
    else if(m->priv->x > m->w - DIGIT_W * NUMBER_WIDTH)
        m->priv->x = m->w - DIGIT_W * NUMBER_WIDTH;
    if(m->priv->y < 0)
        m->priv->y = m->h-DIGIT_H + m->priv->y;
    if(m->priv->y < 0)
        m->priv->y = 0;
    else if(m->priv->y > m->h - DIGIT_H)
        m->priv->y = m->h - DIGIT_H;

    /* Publish module interface */
    m->process_image = m_image;

    return VI3X_RET_OK;
    }

/* Frame processor */
static int m_image(vi3x_module_t *m, vi3x_image_t *image)
    {
    int x;
    unsigned long n;

    x=m->priv->x + DIGIT_W * (NUMBER_WIDTH - 1);
    n=m->frame_n;
    do
        {
        put_digit(image, x, m->priv->y, n % 10);
        x-=DIGIT_W;
        n/=10;
        }
    while(x >= 0 && n > 0);

    return m->out_image(m, image);
    }

/* Digits assumed to have empty last column and last row */
static void put_digit(vi3x_image_t *image, int x, int y, int n)
    {
    int i, j;

    for(i=0; i < DIGIT_H - 1; ++i)
        {
        for(j=0; j < DIGIT_W - 1; ++j)
            {
            if(0 != Digits[n][i][j])
                {
                put_pixel(image, x + j, y + i, VI3X_MAX, VI3X_GREY, VI3X_GREY);
                put_pixel(image, x + j + 1, y + i + 1,
                    VI3X_MIN, VI3X_GREY, VI3X_GREY);
                }
            }
        }
    }

static void put_pixel(vi3x_image_t *image, int x, int y, vi3x_sample_t yy, vi3x_sample_t cb, vi3x_sample_t cr)
    {
    image->yy[image->w*y+x]=yy;
    image->cb[image->w*y+x]=cb;
    image->cr[image->w*y+x]=cr;
    }
