#ifndef _VI3X_TYPES_H_
#define _VI3X_TYPES_H_

#include <stdint.h>
#include <stddef.h>
#include <stdarg.h>

#define VI3X_VER_STR "0.2"

enum VI3X_DELIMS
    {
    VI3X_D_RANGE='@',
    VI3X_D_PAIR='-',
    VI3X_D_LIST=',',
    VI3X_D_END=':',
    VI3X_D_VAL='='
    };
#define VI3X_D_LIST_VAL ",="
#define VI3X_D_END_RANGE ":@"

typedef uint8_t vi3x_sample_t;
#define VI3X_MIN 0
#define VI3X_MAX 255
#define VI3X_GREY 128

enum VI3X_RET
    {
    VI3X_RET_FINISH=-1,
    VI3X_RET_ERR=0,
    VI3X_RET_OK=1
    };

typedef struct vi3x_image_s
    {
    int w;
    int h;
    int count;
    int plane_sz;
    int img_sz;
    vi3x_sample_t *yy;
    vi3x_sample_t *cb;
    vi3x_sample_t *cr;
    int il;
    } vi3x_image_t;

typedef struct vi3x_420_s
    {
    int w;
    int h;
    int yy_count;
    int yy_sz;
    int cw;
    int ch;
    int cc_count;
    int cc_sz;
    int img_sz;
    vi3x_sample_t *yy;
    vi3x_sample_t *cb;
    vi3x_sample_t *cr;
    int il;
    } vi3x_420_t;

typedef struct vi3x_arg_s vi3x_arg_t;
typedef struct vi3x_help_s vi3x_help_t;

typedef void vi3x_arg_help_t(vi3x_arg_t const *arg, vi3x_help_t const *hf);

struct vi3x_arg_s
    {
    char const *name;
    int def;
    size_t offset;
    vi3x_arg_help_t *help;
    };

typedef struct vi3x_s
    {
    vi3x_sample_t *(*init_420)(
        vi3x_420_t *image, int w, int h, vi3x_sample_t *buf);
    vi3x_sample_t *(*init_image)(
        vi3x_image_t *image, int w, int h, vi3x_sample_t *buf);
    void (*copy_image)(vi3x_image_t *dst, vi3x_image_t const *src);
    void (*upsample_mpeg1)(vi3x_image_t *dst, vi3x_420_t const *src);
    void (*upsample_mpeg2)(vi3x_image_t *dst, vi3x_420_t const *src);
    void (*downsample_mpeg1)(vi3x_420_t *dst, vi3x_image_t const *src);
    void (*downsample_mpeg2)(vi3x_420_t *dst, vi3x_image_t const *src);
    void (*fields_to_frame)(vi3x_image_t *img);
    void (*frame_to_fields)(vi3x_image_t *img);
    int (*read)(void *buf, int n);
    int (*write)(void const *buf, int n);
    void (*err)(char const *fmt, ...);
    void (*warn)(char const *fmt, ...);
    void (*info)(char const *fmt, ...);
    void (*info_va)(char const *fmt, va_list ap);
    int (*parse_args)(
        void *params, char const *args, vi3x_arg_t const *map, char const *tag);
    void *(*alloc)(int sz, char const *tag);
    void (*free)(void *ptr);
    } vi3x_t;

struct vi3x_help_s
    {
    void (*help)(char const *fmt, ...);
    void (*help_module)(
        char const *tag,
        char const *ver,
        char const *auth,
        char const *summary,
        char const *help_text,
        vi3x_arg_t const *args
        );
    void (*help_args)(vi3x_arg_t const *map);
    void (*help_param_start)(void);
    void (*help_param_range)(
        char const *name,
        char const *summary,
        char const *help_text,
        int def, int min, int max
        );
    void (*help_param_list)(
        char const *name,
        char const *summary,
        char const *help_text,
        int def, int count, ...
        );
    void (*help_param_bool)(
        char const *name,
        char const *summary,
        char const *help_text,
        int def,
        char const *false_text,
        char const *true_text
        );
    void (*help_module_end)(void);
    };

typedef struct vi3x_module_s vi3x_module_t;
typedef int vi3x_module_f(vi3x_module_t *m);
typedef int vi3x_image_f(vi3x_module_t *m, vi3x_image_t *image);
typedef void vi3x_help_f(vi3x_help_t *h);

struct vi3x_module_s
    {
    /* Initialized by create_module */
    vi3x_t vi3x;
    char const *name;
    vi3x_module_f *init;
    char const *args;
    long frame_n;
    int disabled;
    long *range;
    long *range_ptr;
    /* Initialized by vi3x_init */
    vi3x_image_f *out_image;
    vi3x_module_t *next;
    /* Initialized/modified by module */
    struct m_priv_s *priv;
    int w;
    int h;
    int fps;
    int fps_div;
    vi3x_image_f *process_image;
    vi3x_image_f *flush;
    vi3x_module_f *fini;
    };

#endif /* _VI3X_TYPES_H_ */
